import java.util.Collection;

public interface StationsVelibFacadeI{
  public static final String URL_VELIB_ALL = "http://www.velib.paris.fr/service/carto";
  public static final String URL_VELIB_INFO= "http://www.velib.paris.fr/service/stationdetails/"; //  /number
 
  /** Obtention de la station Vlib  partir de son numro.
   * @param numero le numro de la station par ex. 3011
   * @throws un exception si ce numro est incorrect
   */
  public StationVelib getStation(int numero) throws Exception;
  
  /** Obtention des informations "temps-rl" de la station vlib.
   * En mode dgrad, ce seront les dernires valeurs enregistres.
   * @param s la station vlib
   * @return toutes les informations sur cette station
   */
  public InfoStation getInfoStation(StationVelib s) throws Exception;
  
  /** Obtention de la liste des stations, celle-ci est immutable.
   * L'itrateur retourn par l'appel de la mthode iterator(), lve
   * une exception lors de l'appel d'un retrait par l'appel de la mthode remove
   * @return la liste immuable des stations Vlib
   */
  public Collection<StationVelib> getListeDesStations() throws Exception;
  
  /** Obtention de la distance d'une station  partir de coordonnes GPS.
   * Calcul de la distance extrait de http://www.geodatasource.com/developers/java
   * @param latitude la latitude
   * @param longitude la longitude
   * @param s la station vlib
   * @return la distance en kilomtre entre ce point et la station s, 
   *          0.0 si la station n'existe pas
   */
  public double getDistance(double latitude, double longitude, StationVelib s);
  
  /** Obtention du parser Xml associ.
   * @return le parser Xml choisi
   */
  public XmlParserStationVelibI getParser();
  
  /** Affectation, slection du parser Xml.
   * @param parser le parser Xml
   */
  public void setParser(XmlParserStationVelibI parser);
}
